#
# Copyright 2014, Martin Owens <doctormo@gmail.com>
#
# This file is part of the software inkscape-web, consisting of custom
# code for the Inkscape project's django-based website.
#
# inkscape-web is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# inkscape-web is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with inkscape-web.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Provides template functions as easy to use methods.
"""

__all__ = ['has_template', 'get_template', 'render_template', 'render_directly']

from django.template import TemplateDoesNotExist, TemplateSyntaxError, Template
from django.template.base import VariableDoesNotExist
from django.template.context import Context
from django.template.loader import get_template

def has_template(template_name):
    """We load a template name and return its contents rather than a Template object"""
    try:
        return bool(get_template(template_name))
    except TemplateDoesNotExist:
        return False

def render_template(template_name, context):
    """Render the named template with the given context"""
    template = get_template(template_name)
    return render_directly(template, context)

def render_directly(template, context):
    """Render the template with the given context"""
    if isinstance(template, str):
        template = "{% load i18n %}" + template
        template = Template(template)
    try:
        if isinstance(template, Template):
            return template.render(Context(context))
        return template.render(context)
    except (VariableDoesNotExist, TemplateSyntaxError) as error:
        raise ValueError(str(error))



#
# Copyright 2016, Martin Owens <doctormo@gmail.com>
#
# This file is part of the software inkscape-web, consisting of custom 
# code for the Inkscape project's django-based website.
#
# inkscape-web is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# inkscape-web is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with inkscape-web.  If not, see <http://www.gnu.org/licenses/>.
#
"""
IRC Commands and support for alerts.
"""
import signal

from django.contrib.auth import get_user_model
from django.utils.timezone import now
from django.utils import translation
from django.utils.translation import ugettext as _

from ircbot.base import BotCommand, url

from .models import Message, UserAlert

class TellCommand(BotCommand):
    regex = "tell (\w+) (.+)$"
    is_direct = True

    def run_command(self, context, nick, body):
        from_user = get_user_model().objects.filter(ircnick__iexact=context.nick)
        if from_user.count() != 1:
            return _(u'Your irc nickname must be configured on the website. '
                      'Or it must be the only user with this irc nickname.')

        to_user = get_user_model().objects.filter(ircnick__iexact=nick)
        if to_user.count() > 1:
            return _(u'Too many users have that irc nickname configured.')
        elif to_user.count() == 0:
            return _(u'Can not find user with irc nickname "%(nick)s"') % {'nick': nick}
    
        message = Message.objects.create(subject="From IRC", body=body,
                     sender=from_user.get(), recipient=to_user.get())

        return u'%s: %s' % (context.nick, _('Message Sent'))


class RecieveAlert(BotCommand):
    """This isn't a command, but a signal waiter from the alerts system"""
    def __init__(self, *args, **kw):
        super(RecieveAlert, self).__init__(*args, **kw)
        self.last_alert = now()
        signal.signal(signal.SIGUSR1, self.recieve_alert)

    def recieve_alert(self, signum, frame):
        """
        When an alert is created, we dispatch the alert to the user on irc if they are available.
        """
        for alert in UserAlert.objects.filter(created__gt=self.last_alert):
            self.last_alert = now()
            user = alert.user
            nick = user.ircnick
            if not nick:
                continue
            translation.activate(user.language or 'en')
            self.client.privmsg(nick,
              _("ALERT: %(subject)s ... More info: %(url)s") % { 
                'subject': alert.subject,
                'url': url(alert.get_absolute_url()),
              }
            )





#
# Copyright 2015, Martin Owens <doctormo@gmail.com>
#
# This file is part of the software inkscape-web, consisting of custom 
# code for the Inkscape project's django-based website.
#
# inkscape-web is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# inkscape-web is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with inkscape-web.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Loads all the alert modules and registers everything when ready.
"""

import re
import sys
from importlib import import_module

from django.apps import AppConfig, apps
from django.utils.module_loading import module_has_submodule
from django.db.models.signals import post_migrate

ALERT_MODULE_NAME = 'alert'
IS_TEST = 'test' in sys.argv or 'autotest' in sys.argv
IS_MIGRATE = 'migrate' in sys.argv or 'makemigrations' in sys.argv

class AlertsConfig(AppConfig):
    name = 'alerts'

    def ready(self):
        from alerts.base import BaseAlert

        for app_config in apps.app_configs.values():
            app = app_config.module
            if module_has_submodule(app, ALERT_MODULE_NAME):
                app = app.__name__
                module = import_module("%s.%s" % (app, ALERT_MODULE_NAME))
                for (name, value) in module.__dict__.items():
                    if type(value) is type(BaseAlert) and \
                         issubclass(value, BaseAlert) and \
                         value.__module__ == module.__name__ and \
                         (not value.test_only or IS_TEST):
                        self.register(app, value)

    def register(self, app, cls):
        name = re.sub('(?!^)([A-Z]+)', r'_\1', cls.__name__)
        slug = "{}.{}".format(app, name)[:32].lower()
        try:
            cls(slug, is_test=IS_TEST, is_migrate=IS_MIGRATE) # Singleton
        except Exception as err:
            if any(cmd in sys.argv for cmd in ['runserver', 'shell']):
                sys.stderr.write("Err in alert: %s\n%s\n" % (slug, str(err)))
            raise




#
# Copyright 2015, Martin Owens <doctormo@gmail.com>
#
# This file is part of the software inkscape-web, consisting of custom 
# code for the Inkscape project's django-based website.
#
# inkscape-web is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# inkscape-web is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with inkscape-web.  If not, see <http://www.gnu.org/licenses/>.
#

from django.utils.translation import ugettext_lazy as _

from alerts.base import CreatedAlert
from alerts.models import Message

class MessageAlert(CreatedAlert):
    """Shows overloading of alert signal to process replies as read"""
    name     = _('Personal Message')
    desc     = _('Another user has sent you a personal message.')
    info     = _("When another user on the website sends you a message directly, this is called a <strong>Personal Message</strong>. By default these messages are emailed to you.")
    alert_user = 'recipient'
    sender   = Message

    subject       = "{{ instance.subject }}"
    email_subject = "Message from User: {{ instance.subject }}"
    object_name   = "User's personal messages: {{ object }}"

    default_email = True
    default_irc = False

    subscribe_all = False
    subscribe_any = False
    subscribe_own = True

    def call(self, sender, instance, **kwargs):
        """Marks the message we're replying to as viewed"""
        # This can cause an integrity error if called from a loaddata atomic block
        if super().call(instance=instance, **kwargs):
            if instance.reply_to:
                instance.reply_to.alerts.all().view_all()


class TestMessageAlert(MessageAlert):
    """Used in tests to get different results"""
    object_name   = "Each message themselves for testing"
    alert_user = 'sender'
    test_only = True
    related_name = 'test_alerts'





# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('alerts', '0002_auto_20150716_0346'),
    ]

    operations = [
        migrations.AlterField(
            model_name='useralertobject',
            name='o_id',
            field=models.PositiveIntegerField(default=1),
            preserve_default=False,
        ),
    ]



#
# Copyright 2016, Martin Owens <doctormo@gmail.com>
#
# This file is part of the software inkscape-web, consisting of custom 
# code for the Inkscape project's django-based website.
#
# inkscape-web is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# inkscape-web is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with inkscape-web.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Sync all configured mailing lists and insert into forums.
"""

from collections import defaultdict

from django.core.management.base import BaseCommand
from django.utils.translation import ugettext_lazy as _

from alerts.models import UserAlert, UserAlertSetting

class Command(BaseCommand):
    help = "Batches all messages into one email for users with those settings"
    def add_arguments(self, parser):
        parser.add_argument(
            '--mode',
            action='store',
            dest='mode',
            default='D',
            help='Name of the mode, either "D", "W" or "M"')
        return parser

    def handle(self, mode="D", **kw):
        # We first combine all the batch settings of the same type.
        # So user 5 with alert types 2 and 18 both set to D (daily)
        # will be batched together in the second step.
        settings = defaultdict(list)
        for user_id, alert_id in UserAlertSetting.objects\
                .filter(batch=mode, email=True)\
                .values_list('user_id', 'alert_id'):
            settings[user_id].append(alert_id)

        for user_id, alert_ids in settings.items():
            UserAlert.objects.filter(
                viewed__isnull=True, # Not Viewed
                deleted__isnull=True, # Not Deleted
            ).send_batch_email(
                batch_mode=mode,
                user_id=user_id, # Belong to this user
                alert_ids=alert_ids, # For this type
            )





#
# Copyright 2014, Martin Owens <doctormo@gmail.com>
#
# This file is part of the software inkscape-web, consisting of custom 
# code for the Inkscape project's django-based website.
#
# inkscape-web is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# inkscape-web is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with inkscape-web.  If not, see <http://www.gnu.org/licenses/>.
#
from django.contrib.admin import ModelAdmin, StackedInline, site

from .models import *

class ObjectInline(StackedInline):
    model = UserAlertObject
    extra = 0

class ValueInline(StackedInline):
    model = UserAlertValue
    extra = 0

class UserAlertAdmin(ModelAdmin):
    inlines = [ObjectInline, ValueInline]

site.register(AlertType)
site.register(UserAlert, UserAlertAdmin)
site.register(UserAlertSetting)
site.register(AlertSubscription)

site.register(Message)




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('alerts', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='alerttype',
            name='private',
            field=models.BooleanField(default=False),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='alerttype',
            name='category',
            field=models.CharField(default=b'?', max_length=1, verbose_name='Category', choices=[(b'?', b'Unknown'), (b'U', b'User to User'), (b'S', b'System to User'), (b'A', b'Admin to User'), (b'P', b'User to Admin'), (b'T', b'System to Translator')]),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
#
# Copyright 2014, Martin Owens <doctormo@gmail.com>
#
# This file is part of the software inkscape-web, consisting of custom
# code for the Inkscape project's django-based website.
#
# inkscape-web is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# inkscape-web is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with inkscape-web.  If not, see <http://www.gnu.org/licenses/>.
#
import json

from django.http import Http404, HttpResponse, HttpResponseRedirect, JsonResponse
from django.shortcuts import get_object_or_404, render_to_response
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.decorators import login_required
from django.contrib import messages
from django.template import RequestContext
from django.core.urlresolvers import reverse
from django.views.generic import ListView, DeleteView, CreateView, UpdateView,\
        TemplateView, View
from django.views.generic.detail import SingleObjectMixin
from django.contrib.auth import get_user_model
from django.db import models

from .forms import MessageForm, SettingsFormSet
from .mixins import NeverCacheMixin, UserRequiredMixin, UserMixin, OwnerRequiredMixin
from .models import UserAlert, Message, UserAlertSetting, AlertType, AlertSubscription

class AlertsJson(NeverCacheMixin, UserRequiredMixin, View):
    def get(self, request):
        alerts = request.user.alerts.all()
        context = {
           'types': tuple(AlertType.objects.values()) or None,
           'new': tuple(alerts.new.values()) or None,
        }
        return JsonResponse(context)

class IsNull(models.Func):
    template = "%(expressions)s IS NULL"

class AlertList(NeverCacheMixin, OwnerRequiredMixin, ListView):
    """Shows a list of user alerts, user only sees their own"""
    paginate_by = 15

    def get_queryset(self, **kwargs):
        qs = self.request.user.alerts.all().visible
        self.breadcrumb_root = self.request.user
        if 'slug' in self.kwargs:
            self.parent = get_object_or_404(AlertType, slug=self.kwargs['slug'])
            qs = qs.filter(alert__slug=self.kwargs['slug'])
        else:
            self.parent = (reverse('alerts'), _('Inbox'))

        if 'new' in self.request.GET:
            self.title = _("New")
            qs = qs.filter(viewed__isnull=True, deleted__isnull=True)
        qs = qs.annotate(not_viewed=IsNull('viewed'))
        return qs.order_by('-not_viewed', 'viewed', '-created')

    def get_context_data(self, **data):
        data = super(AlertList, self).get_context_data(**data)
        # Disable alert vew
        data['alerts'] = True
        return data




class MarkViewed(NeverCacheMixin, OwnerRequiredMixin, View, SingleObjectMixin):
    model = UserAlert
    function = 'view'

    def get(self, request, *args, **kwargs):
        obj = self.get_object()
        getattr(obj, self.function)()
        return HttpResponse(json.dumps({self.function: [obj.pk]}))


class MarkAllViewed(AlertList):
    function = 'view'

    def get(self, request, *args, **kwargs):
        objs = self.get_queryset()
        # This list() MUST happen before the function otherwise delete
        # will return an empty list and fail to update the html.
        pks = list(objs.values_list('pk', flat=True))

        getattr(objs, self.function + '_all')()
        return HttpResponse(json.dumps({self.function: list(pks)}))
 

class MarkDeleted(MarkViewed):
    function = 'delete'

class MarkAllDeleted(MarkAllViewed):
    function = 'delete'


class Subscribe(NeverCacheMixin, UserRequiredMixin, CreateView):
    model = AlertSubscription
    fields = [] # Everything is in url or context

    def get(self, request, **kwargs):
        if request.GET.get('post', False):
            return self.post(request, **kwargs)
        return super(Subscribe, self).get(request, **kwargs)

    def get_context_data(self, **kwargs):
        data = super(Subscribe, self).get_context_data(**kwargs)
        self.breadcrumb_root = self.request.user
        data['alert'] = get_object_or_404(AlertType, slug=self.kwargs['slug'])
        data['object'] = data['alert']
        if 'pk' in self.kwargs:
            try:
                subscription = data['alert'].get_object(pk=self.kwargs['pk'])
            except data['alert'].sender.DoesNotExist:
                raise Http404("Alert subscription doesn't exist")

            data['object_name'] = data['alert'].get_object_name(subscription)
            data['title'] = _('Subscribe to %(object_name)s') % data
        else:
            data['alert_name'] = data['alert'].name
            data['title'] = _('Subscribe to All %(alert_name)s') % data
        return data

    def post(self, request, **kwargs):
        self.object = None
        data = self.get_context_data(**kwargs)
        kwargs = dict(user=request.user, target=self.kwargs.get('pk', None))
        (self.object, created, previous) = data['alert'].subscriptions.get_or_create(**kwargs)
        if previous:
            messages.warning(request,
                             _("Deleted %d previous subscription(s) (superseded)") % previous)
        if created:
            messages.info(request, _('Subscription created!'))
        else:
            messages.warning(request, _('Already subscribed to this!'))

        next_url = request.GET.get('next', request.POST.get('next', None))
        if not next_url:
            next_url = reverse('alert.settings') + '?tab=' + data['alert'].slug

        return HttpResponseRedirect(next_url)


class Unsubscribe(NeverCacheMixin, OwnerRequiredMixin, DeleteView):
    model = AlertSubscription

    def get(self, request, **kwargs):
        if request.GET.get('post', False):
            return self.post(request, **kwargs)
        return super(Subscribe, self).get(request, **kwargs)

    def get_success_url(self):
        next_url = self.request.GET.get('next', self.request.POST.get('next', None))
        if not next_url:
            next_url = reverse('alert.settings')
        return next_url

    def get_context_data(self, **kwargs):
        data = super(Unsubscribe, self).get_context_data(**kwargs)
        self.breadcrumb_root = self.request.user
        data['alert'] = AlertType.objects.get(slug=self.kwargs['slug'])
        data['object'] = data['alert']
        data['delete'] = True
        if 'pk' in self.kwargs:
            subscription = data['alert'].get_object(pk=self.kwargs['pk'])
            data['object_name'] = data['alert'].get_object_name(subscription)
            data['title'] = _('Unsubscribe from %(object_name)s') % data
        else:
            data['alert_name'] = data['alert'].name
            data['title'] = _('Unsubscribe from All')
        return data
    
    def get_object(self):
        if 'slug' in self.kwargs:
            alert = get_object_or_404(AlertType, slug=self.kwargs['slug'])
            kw = dict(user=self.request.user)
            if 'pk' in self.kwargs:
                kw['target'] = self.kwargs['pk']
            else:
                kw['target__isnull'] = True
            return get_object_or_404(alert.subscriptions, **kw)
        return super(Unsubscribe, self).get_object()


class SettingsList(NeverCacheMixin, UserMixin, UpdateView):
    title = _('Notification Settings')
    form_class = SettingsFormSet
    template_name = 'alerts/alertsettings.html'

    def get_context_data(self, **data):
        data = super(SettingsList, self).get_context_data(**data)
        data['formset'] = data.pop('form')
        return data

    def get_success_url(self):
        return reverse('alert.settings')


class SentMessages(NeverCacheMixin, UserRequiredMixin, ListView):
    model = Message
    title = _('Outbox')
    paginate_by = 15

    def get_queryset(self):
        return self.request.user.sent_messages.all()

    def get_parent(self):
        return self.request.user


class CreateMessage(NeverCacheMixin, UserRequiredMixin, CreateView):
    model = Message
    form_class = MessageForm

    def form_valid(self, form):
        obj = form.save(commit=False)
        obj.sender = self.request.user
        obj.save()
        messages.info(self.request, _('Message Sent to %s') % str(obj.recipient))
        return HttpResponseRedirect(self.get_success_url())

    def get_success_url(self):
        return reverse('my_profile')

    def get_reply_to(self):
        if 'pk' in self.kwargs:
            # If we ever want to restrict who can reply, do it here first.
            return get_object_or_404(self.model, pk=self.kwargs['pk'], recipient=self.request.user)

    def get_initial(self):
        """Add reply to subject initial data"""
        initial = super(CreateMessage, self).get_initial()
        self.recipient = get_object_or_404(get_user_model(), username=self.kwargs.get('username',''))
        rto = self.get_reply_to()
        if rto:
            initial['subject'] = rto.subject
            if not rto.reply_to:
                initial['subject'] = "Re: " + initial['subject']
        if 'pk' in self.kwargs:
            initial['reply_to'] = self.kwargs['pk']
        initial['recipient'] = self.recipient.pk
        return initial

    def get_context_data(self, **data):
        """Add reply to message object to template output"""
        data = super(CreateMessage, self).get_context_data(**data)
        data['reply_to'] = self.get_reply_to()
        data['recipient'] = self.recipient
        data['object_list'] = Message.objects.all()
        data['object_list'].parent = self.recipient

        data['title'] = _("Send New Message")
        if data['reply_to']:
            data['title'] = _("Send Reply Message")

        return data




# -*- coding: utf-8 -*-
#
# Copyright 2016, Martin Owens <doctormo@gmail.com>
#
# This file is part of the software inkscape-web, consisting of custom 
# code for the Inkscape project's django-based website.
#
# inkscape-web is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# inkscape-web is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with inkscape-web.  If not, see <http://www.gnu.org/licenses/>.
#

from django.core.exceptions import PermissionDenied
from django.http import HttpResponseRedirect
from django.conf import settings

from django.utils.decorators import method_decorator
from django.views.decorators.cache import cache_page, never_cache
from django.views.generic.detail import SingleObjectMixin

class NeverCacheMixin(object):
    @method_decorator(never_cache)
    def dispatch(self, *args, **kwargs):
        return super(NeverCacheMixin, self).dispatch(*args, **kwargs)

class UserRequiredMixin(object):
    def is_authorised(self, user):
        return True

    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_authenticated():
            return HttpResponseRedirect(settings.LOGIN_URL)
        if not self.is_authorised(request.user):
            raise PermissionDenied()
        return super(UserRequiredMixin, self).dispatch(request, *args, **kwargs)

class UserMixin(UserRequiredMixin):
    get_object = lambda self: self.request.user

class OwnerRequiredMixin(UserRequiredMixin):
    user_field = 'user'

    def is_authorised(self, user):
        if isinstance(self, SingleObjectMixin):
            return getattr(self.get_object(), self.user_field) == user

        # For lists of items
        qs = self.get_queryset()
        kw = {self.user_field: user}
        return qs.exclude(**kw).count() == 0
        




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('alerts', '0006_auto_20170107_2355'),
    ]

    operations = [
        migrations.AddField(
            model_name='useralertsetting',
            name='filter_value',
            field=models.CharField(help_text='An extra filter variable which can be different for each alert.', max_length=255, null=True, blank=True),
        ),
    ]



#
# Copyright 2014-2017, Martin Owens <doctormo@gmail.com>
#
# django-boxed-alerts is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-boxed-alerts is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with django-boxed-alerts.  If not, see <http://www.gnu.org/licenses/>.
#
"""Record alerts in the database"""

from collections import defaultdict

from django.contrib.auth import get_user_model
from django.conf import settings

from django.db.models import (
    Model, Manager, QuerySet, Q, TextField, CharField,
    ForeignKey, BooleanField, DateTimeField, PositiveIntegerField,
)

from django.utils.translation import ugettext_lazy as _
from django.utils.timezone import now

from django.contrib.auth.models import Group
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType

from django.core.urlresolvers import reverse
from django.core.validators import MaxLengthValidator
from django.core.mail.message import EmailMultiAlternatives
from django.utils import translation

from alerts.template_tools import render_template

BATCH_MODES = (
    (None, _("Instant")),
    ("D", _("Daily")),
    ("W", _("Weekly")),
    ("M", _("Monthly")),
)


class AlertTypeManager(Manager):
    """Manage alert types, ensuring only one type per base is created in the db"""
    def get_by_natural_key(self, slug):
        """Returns the natural key for AlertTypes"""
        return self.get(slug=slug)

    def get_or_create(self, values=None, **kwargs):
        """Allow the AlertType to be updated with code values
           (should only be run on new code load or server restart)"""
        if values and 'defaults' not in kwargs:
            kwargs['defaults'] = values
        (obj, created) = super(AlertTypeManager, self).get_or_create(**kwargs)
        if not created and values:
            self.filter(pk=obj.pk).update(**values)
        return (obj, created)

    def get_queryset(self):
        """ALWAYS limit alert types to those available"""
        from alerts.base import BaseAlert
        qset = super(AlertTypeManager, self).get_queryset()
        return qset.filter(slug__in=list(BaseAlert.instances))


class AlertType(Model):
    """All Possible messages that users can receive, acts as a template"""
    slug = CharField(_("URL Slug"), max_length=128, unique=True)
    group = ForeignKey(Group, verbose_name=_("Limit to Group"), blank=True, null=True)

    created = DateTimeField(_("Created Date"), auto_now_add=now)
    enabled = BooleanField(default=False)

    # These get copied into UserAlertSettings for this alert
    default_email = BooleanField(default=False)
    default_batch = CharField(max_length=1, choices=BATCH_MODES, blank=True, null=True)
    default_irc = BooleanField(default=False)

    objects = AlertTypeManager()

    breadcrumb_parent = staticmethod(lambda self: (reverse('alerts'), _('Inbox')))

    @property
    def _alerter(self):
        # Late import to stop loop import
        from alerts.base import BaseAlert
        return BaseAlert.instances[self.slug].connect_signals()

    def __getattr__(self, name):
        if hasattr(self._alerter, name):
            return getattr(self._alerter, name)
        return getattr(super(AlertType, self), name)

    def send_to(self, user, auth=None, **kwargs):
        """Creates a new alert for a certain user of this type.

         user   - The user this alert should be addressed to. (required)
                  can also be a group, where upon each member of the group
                  will be messaged (be careful with this).
         auth   - Super user to authorise the sending of an alert to everyone
         kwargs - Dictionary of objects used in the rendering of the subject
                  and body templates.

         Returns new UserAlert object or None if user isn't allowed this alert.

        """
        if not self.enabled:
            return []
        if isinstance(user, Group):
            users = user.users
        elif isinstance(user, get_user_model()):
            users = [user]
        elif user == 'all':
            if not isinstance(auth, get_user_model()) or not auth.is_superuser:
                raise ValueError("You are not authorized to send an alert to everyone.")
            users = get_user_model().objects.all()
        else:
            raise ValueError("You must specify a user or group to send an alert to.")

        return [self._send_to(real_user, **kwargs) for real_user in users]

    def _send_to(self, user, **kwargs):
        if self.enabled and (not self.group or self.group in user.groups):
            if 'instance' in kwargs and 'once' in kwargs:
                # Check if the instance has already been issued to this user's alerts.
                i = kwargs['instance']
                existing = UserAlert.objects.filter(
                    user=user, alert=self,
                    objs__o_id=i.pk, objs__name='instance',
                    deleted__isnull=True, viewed__isnull=True)
                if existing.count():
                    return None
            alert = UserAlert(user=user, alert=self)
            alert.save()
            for (key, value) in kwargs.items():
                alert.add_value(key, value)
            # Do this after saving objects and values so email can use them.
            if alert.config.batch is None:
                alert.send_email(context_data=kwargs)
            alert.send_irc_msg()
            return alert
        return None

    def subscribe_url(self, obj=None):
        """Return the url to subscribe to the given object"""
        return self.alert_url('subscribe', pk=(obj and obj.pk), slug=self.slug)

    def unsubscribe_url(self, obj=None):
        """Return the url to unsubscribe to the given object"""
        return self.alert_url('unsubscribe', pk=(obj and obj.pk), slug=self.slug)

    def get_absolute_url(self):
        """Return the url to see alert type details"""
        return self.alert_url('category', slug=self.slug)

    @staticmethod
    def alert_url(view, **kwargs):
        """Returns a standard url for alerts"""
        kwargs = dict((a, b) for (a, b) in kwargs.items() if b is not None)
        return reverse('alert.' + view, kwargs=kwargs)

    def __str__(self):
        try:
            return str(self.name)
        except AttributeError:
            return "Orphan (%s)" % self.slug


class SettingsQuerySet(QuerySet):
    """Provide easy ways of accessing lists of user's alert settings"""
    def get(self, *args, **kwargs):
        """Get the specific setting, creating one if it doesn't exist"""
        hint = self._hints.get('instance', None)
        user = kwargs.get('user', hint)
        alert = kwargs.get('alert', hint)

        try:
            return super().get(*args, **kwargs)
        except self.model.DoesNotExist:
            if not isinstance(user, get_user_model()) \
              or not isinstance(alert, AlertType):
                raise

        # Create a blank, not-saved model containing the default settings.
        return self.model(
            user_id=user.pk,
            alert_id=alert.pk,
            owner=alert.subscribe_own,
            email=alert.default_email,
            irc=alert.default_irc,
            batch=alert.default_batch,
        )

    def for_user(self, user):
        """Return the settings for a specific user"""
        alert = self._hints.get('instance', None)
        return self.get(user=user, alert=alert)



class UserAlertSetting(Model):
    """Allow users to configure how they access alerts on the website"""
    user = ForeignKey(settings.AUTH_USER_MODEL, related_name='alert_settings')
    alert = ForeignKey(AlertType, related_name='settings')
    owner = BooleanField(_("Subscribe to Self"), default=True, \
        help_text=_("Send the alert if it relates directly to me or my group."))

    email = BooleanField(_("Send Email Alert"), default=False, \
        help_text=_("Send the alert to the account email address."))
    irc = BooleanField(_("Send IRC Alert"), default=False, \
        help_text=_("If online, this alert will be sent to my irc nickname."))
    batch = CharField(_("Batch Alerts"), max_length=1, \
        choices=BATCH_MODES, blank=True, null=True, \
        help_text=_("Save all alerts and send as one email, only affects email alerts."))
    filter_value = CharField(max_length=255, blank=True, null=True, \
        help_text=_("An extra filter variable which can be different for each alert."))

    objects = SettingsQuerySet.as_manager()

    slug = property(lambda self: self.alert.slug)

    def __str__(self):
        return str(self.alert)

    @property
    def subscriptions(self):
        """Returns related subscriptions based on the user and alert"""
        return self.alert.subscriptions.filter(user=self.user)

    def is_default(self):
        """Return true if the settings are just the default settings"""
        return self.owner != self.alert.subscribe_own \
            or self.email != self.alert.default_email \
            or self.batch != self.alert.default_batch \
            or self.irc != self.alert.default_irc \
            or self.filter_value not in [None, '']

    def save(self, force_insert=False, force_update=False, using=None, update_fields=None):
        """Do not save default settings if never saved before"""
        if self.pk and not self.is_default():
            super().save(force_insert=False, force_update=False, using=None, update_fields=None)


class UserAlertQuerySet(QuerySet):
    """Provide ways to listing a user's alerts"""
    def serialise(self):
        """Turn the list of alerts into a list of dictionaries"""
        # It's not possible to use qs.values(...) because we need
        # to return the subject and body lines too which are template
        # rendered and not just values in the database.
        return [{
            'id': item.pk,
            'subject': item.subject,
            'body': item.body,
            'created': item.created,
            'viewed': item.viewed,
            'deleted': item.deleted,
            'alert': item.alert.pk,
        } for item in self]

    @property
    def new(self):
        """Return all new (unread) alerts"""
        return self.filter(viewed__isnull=True, deleted__isnull=True)

    @property
    def visible(self):
        """Return a list of visible alerts"""
        return self.filter(deleted__isnull=True)

    def view_all(self):
        """Mark all new alerts as visible"""
        return self.new.update(viewed=now())

    def delete_all(self):
        """Delete all visible alerts"""
        return self.visible.update(deleted=now())

    def send_batch_email(self, batch_mode, user_id, alert_ids):
        """Sends all the selected messages out as a batched email"""
        qset = self.filter(user_id=user_id, alert_id__in=alert_ids)
        if qset.count() == 0:
            return False
        if qset.count() == 1:
            return self[0].send_email()

        user = qset[0].user
        context_data = {
            'mode': batch_mode,
            'types': len(alert_ids),
            'count': qset.count(),
            'alerts': qset,
            'user': user
        }

        with translation.override(user.language):
            subject = {
                'D': _("Inkscape Daily Notifications (%d)"),
                'W': _("Inkscape Weekly Notifications (%d)"),
                'M': _("Inkscape Monthly Notifications (%d)"),
            }[batch_mode] % qset.count()

            template = "alerts/alert/batch_email.txt"

            kwargs = {
                'subject': subject,
                'body': render_template(template, context_data),
                'to': [user.email],
            }
            if EmailMultiAlternatives(**kwargs).send(True):
                qset.view_all()
        return True

class UserAlertManager(Manager):
    """Provide easy access to alert lists for users"""
    _queryset_class = UserAlertQuerySet

    def get_queryset(self):
        """Returns alerts for a user, butonly currently live ones"""
        queryset = super(UserAlertManager, self).get_queryset()

        from alerts.base import BaseAlert
        # Limit query to currently live/available alerts
        queryset = queryset.filter(alert__slug__in=list(BaseAlert.instances))

        if getattr(self, 'target', None) is not None:
            ctype = UserAlertObject.target.get_content_type(obj=self.target)
            queryset = queryset.filter(objs__table=ctype, objs__o_id=self.target.pk)

        if getattr(self, 'alert_type', None) is not None:
            queryset = queryset.filter(alert=self.alert_type)

        return queryset.order_by('-created')

    @property
    def parent(self):
        """Returns what object is the parent, ususally the user object"""
        if 'user__exact' in self.core_filters:
            return self.core_filters['user__exact']
        return None


class UserAlert(Model):
    """A single alert for a specific user"""
    user = ForeignKey(settings.AUTH_USER_MODEL, related_name='alerts')
    alert = ForeignKey(AlertType, related_name='sent')

    created = DateTimeField(auto_now_add=True)
    viewed = DateTimeField(blank=True, null=True)
    deleted = DateTimeField(blank=True, null=True)

    objects = UserAlertManager()

    subject = property(lambda self: self.alert.get_subject(self.data))
    body = property(lambda self: self.alert.get_body(self.data))
    email_body = property(lambda self: self.alert.get_email_body(self.data))

    def view(self):
        """Set the alert as viewed"""
        if not self.viewed:
            self.viewed = now()
            self.save()

    def delete(self):
        """Set the alert as deleted"""
        if not self.deleted:
            self.deleted = now()
            self.save()

    def is_hidden(self):
        """Return true if the alert is viewed or deleted"""
        return self.viewed or self.deleted

    @property
    def config(self):
        """Return the settings for this alert's type and this user"""
        # This should auto-create but not save.
        return UserAlertSetting.objects.get(user=self.user, alert=self.alert)

    def __str__(self):
        return "<UserAlert %s>" % str(self.created)

    @staticmethod
    def get_absolute_url():
        """Return a link to all alerts for this user"""
        return reverse('alerts')

    @property
    def data(self):
        """Return a dictionary of fields for this alert"""
        ret = defaultdict(list, alert=self, site=settings.SITE_ROOT)
        for item in list(self.objs.all()) + list(self.values.all()):
            try:
                target = item.target
            except AttributeError:
                target = None

            if item.name[0] == '@':
                ret[item.name[1:]+'_list'].append(target)
            else:
                ret[item.name] = target
        return ret

    def add_value(self, name, value):
        """Add the named value to this alert's value list"""
        if isinstance(value, (tuple, list)) and name[0] != '@':
            for x in value:
                self.add_value('@' + name, x)
        elif isinstance(value, Model):
            UserAlertObject(alert=self, name=name, target=value).save()
        else:
            UserAlertValue(alert=self, name=name, target=str(value)).save()

    def send_irc_msg(self):
        """Send this alert message over IRC if it's available"""
        if not self.config.irc:
            return False
        return self.alert.send_irc_msg(self)

    def send_email(self, **kwargs):
        """Send alert email is user's own language"""
        if not self.config.email:
            return False
        with translation.override(self.user.language or 'en'):
            data = self.data.copy()
            data.update(kwargs.pop('context_data', {}))
            if self.alert.send_email(self.user.email, data, **kwargs):
                self.view()
        return True


class UserAlertObject(Model):
    """User alert value, linked to another database object"""
    alert = ForeignKey(UserAlert, related_name='objs')
    name = CharField(max_length=128)

    table = ForeignKey(ContentType, blank=True, null=True)
    o_id = PositiveIntegerField()
    target = GenericForeignKey('table', 'o_id')

    def __str__(self):
        return "AlertObject %s=%s" % (self.name, str(self.o_id))


class UserAlertValue(Model):
    """User alert value, scalar value stored as a string"""
    alert = ForeignKey(UserAlert, related_name='values')
    name = CharField(max_length=128)
    target = CharField(max_length=255)

    def __str__(self):
        return "AlertValue %s=%s" % (self.name, self.target)


class SubscriptionQuerySet(QuerySet):
    """Access a list of subscriptions for a user"""
    def get_or_create(self, target=None, **kwargs):
        """Handle the match between a null target and non-null targets"""
        deleted = 0
        if target:
            # if subscription with no target exists, return that.
            try:
                obj = self.get(target__isnull=True, **kwargs)
            except AlertSubscription.DoesNotExist:
                return super().get_or_create(target=target, **kwargs) + (0,)
            else:
                return (obj, False, deleted)

        (obj, created) = super().get_or_create(target=target, **kwargs)
        if created:
            # replace all other existing subscriptions with this one.
            to_delete = self.filter(target__isnull=False, **kwargs)
            deleted = to_delete.count()
            to_delete.delete()
        return (obj, created, deleted)

    def is_subscribed(self, target=None, directly=False):
        """Returns true if the user is already subscribed to this"""
        if target is None:
            return bool(self.filter(target__isnull=True).count())
        if directly:
            return bool(self.filter(target=target.pk).count())
        return bool(self.are_subscribed(target))

    def are_subscribed(self, target):
        """Returns both target sepcific and class based subscription"""
        query = Q(target__isnull=True)
        if target is not None:
            query |= Q(target=target.pk)
        return dict(self.filter(query).values_list('target', 'pk'))


class AlertSubscriptionManager(Manager.from_queryset(SubscriptionQuerySet)):
    """Manage lists of user subscriptions"""
    def get_queryset(self):
        """Return alert subscriptions with the right targeting"""
        queryset = super(AlertSubscriptionManager, self).get_queryset()

        if getattr(self, 'alert_type', None) is not None:
            queryset = queryset.filter(alert=self.alert_type)
        if hasattr(self, 'target'):
            if self.target is not None:
                queryset = queryset.filter(target=self.target.pk)
            else:
                queryset = queryset.filter(target__isnull=True)
        return queryset


class AlertSubscription(Model):
    """A subscription to a object or list of object's alerts"""
    alert = ForeignKey(AlertType, related_name='subscriptions')
    user = ForeignKey(settings.AUTH_USER_MODEL, related_name='alert_subscriptions')
    target = PositiveIntegerField(_("Object ID"), blank=True, null=True)

    objects = AlertSubscriptionManager()

    def object(self):
        """Return the associated object (alert target)"""
        return self.alert.get_object(pk=self.target)

    def __str__(self):
        return "%s Subscription to %s" % (str(self.user), str(self.alert))


# -------- Start Example App -------- #

class MessageQuerySet(QuerySet):
    """Provide links to the user interface for lists of messages"""
    breadcrumb_name = staticmethod(lambda: _("Messages"))
    get_absolute_url = staticmethod(lambda: reverse("alerts"))

class Message(Model):
    """
    User messages are a simple alert example system allowing
    users to send messages between each other.
    """
    sender = ForeignKey(settings.AUTH_USER_MODEL, related_name="sent_messages")
    recipient = ForeignKey(settings.AUTH_USER_MODEL, related_name="messages")
    reply_to = ForeignKey('self', related_name="replies", null=True, blank=True)
    subject = CharField(max_length=128)
    body = TextField(_("Message Body"),\
        validators=[MaxLengthValidator(8192)], null=True, blank=True)
    created = DateTimeField(default=now)

    objects = MessageQuerySet.as_manager()

    def get_root(self, children=None):
        """Returns the root message for the thread"""
        children = children or tuple()
        if self.reply_to:
            # Break infinate root-to-branch loop in tree
            if id(self) in children:
                return self
            return Message.get_root(self.reply_to, children+(id(self),))
        return self

    def __str__(self):
        return "Message from %s" % str(self.sender)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('alerts', '0005_auto_20161108_1700'),
    ]

    operations = [
        migrations.AlterField(
            model_name='useralertsetting',
            name='batch',
            field=models.CharField(choices=[(None, 'Instant'), (b'D', 'Daily'), (b'W', 'Weekly'), (b'M', 'Monthly')], max_length=1, blank=True, help_text='Save all alerts and send as one email, only affects email alerts.', null=True, verbose_name='Batch Alerts'),
        ),
        migrations.AlterField(
            model_name='useralertsetting',
            name='owner',
            field=models.BooleanField(default=True, help_text='Send the alert if it relates directly to me or my group.', verbose_name='Subscribe to Self'),
        ),
    ]



#
# Copyright 2013, Martin Owens <doctormo@gmail.com>
#
# This file is part of the software inkscape-web, consisting of custom
# code for the Inkscape project's django-based website.
#
# inkscape-web is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# inkscape-web is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with inkscape-web.  If not, see <http://www.gnu.org/licenses/>.
#
"""Alerts urls"""

from django.conf.urls import url, include
from .views import (
    SettingsList, MarkAllViewed, MarkAllDeleted, MarkViewed,
    MarkDeleted, AlertList, AlertsJson, SentMessages, Subscribe, Unsubscribe
)

def url_tree(regex, *urls):
    """Provide a way to extend patterns easily"""
    class UrlTwig(object): # pylint: disable=too-few-public-methods, missing-docstring
        urlpatterns = urls
    return url(regex, include(UrlTwig))

urlpatterns = [ # pylint: disable=invalid-name
    url(r'^$', AlertList.as_view(), name="alerts"),
    url(r'^json/$', AlertsJson.as_view(), name="alerts.json"),
    url(r'^view/$', MarkAllViewed.as_view(), name="alert.view"),
    url(r'^outbox/$', SentMessages.as_view(), name="message.outbox"),
    url(r'^delete/$', MarkAllDeleted.as_view(), name='alert.delete'),
    url(r'^settings/$', SettingsList.as_view(), name='alert.settings'),

    url_tree(
        r'^(?P<pk>\d+)/',
        url(r'^delete/', MarkDeleted.as_view(), name='alert.delete'),
        url(r'^view/', MarkViewed.as_view(), name="alert.view"),
    ),
    url_tree(
        r'^(?P<slug>[^\/]+)/',
        url(r'^$', AlertList.as_view(), name="alert.category"),
        url_tree(
            r'^subscribe/',
            url(r'^$', Subscribe.as_view(), name='alert.subscribe'),
            url(r'^(?P<pk>\d+)/$', Subscribe.as_view(), name='alert.subscribe'),
        ),
        url_tree(
            r'^unsubscribe/',
            url(r'^$', Unsubscribe.as_view(), name='alert.unsubscribe'),
            url(r'^(?P<pk>\d+)/$', Unsubscribe.as_view(), name='alert.unsubscribe'),
        ),
    ),
]



#
# Copyright 2017, Martin Owens <doctormo@gmail.com>
#
# django-boxed-alerts is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-boxed-alerts is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with django-boxed-alerts.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Checkbox dropdown selection box.
"""

from django.forms.widgets import SelectMultiple
from django.forms.fields import MultipleChoiceField

class CheckdropSelectMultiple(SelectMultiple):

    class Media:
        css = {
            'all': ('css/sumoselect.css',)
        }
        js = ('js/sumoselect.min.js',)

    def render(self, name, *args, **kw):
        ret = super(CheckdropSelectMultiple, self).render(name, *args, **kw)
        ret += """
        <script type="text/javascript">$('#id_%s').SumoSelect();</script>
        """ % name
        return ret


class MultipleCheckboxField(MultipleChoiceField):
    widget = CheckdropSelectMultiple

    def clean(self, value):
        lst = super(MultipleCheckboxField, self).clean(value)
        return "|%s|" % '|'.join(lst)

    def prepare_value(self, value):
        if isinstance(value, (list, tuple)):
            return value
        return str(value).strip('|').split('|')




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.utils.timezone
from django.conf import settings
import django.core.validators

from django.db.migrations.operations.base import Operation
class DeleteIfExists(Operation):
    """Delete a table if exists (ignores model)"""
    def __init__(self, table_name):
        self.table_name = table_name

    def state_forwards(self, app_label, state):
        pass

    def database_forwards(self, app_label, schema_editor, from_state, to_state): 
        if schema_editor.connection.vendor == 'mysql':
            schema_editor.execute("SET FOREIGN_KEY_CHECKS=0")
            try:
                schema_editor.execute("DROP TABLE IF EXISTS %(table)s CASCADE" % {
                    "table": schema_editor.quote_name(self.table_name),
                })
            except Exception as error:
                if 'Warning' not in type(error).__name__:
                    raise
            schema_editor.execute("SET FOREIGN_KEY_CHECKS=1")
        else:
            schema_editor.execute("DROP TABLE IF EXISTS %(table)s" % {
                "table": schema_editor.quote_name(self.table_name),
            })

    def database_backwards(self, app_label, schema_editor, from_state, to_state):
        pass

    def describe(self):
        return "Delete %s table if exists" % self.table_name



class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        #DeleteIfExists('alerts_alertsubscription'),
        migrations.CreateModel(
            name='AlertSubscription',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('target', models.PositiveIntegerField(null=True, verbose_name='Object ID', blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        #DeleteIfExists('alerts_alerttype'),
        migrations.CreateModel(
            name='AlertType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('slug', models.CharField(max_length=32, verbose_name='URL Slug')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created Date')),
                ('category', models.CharField(default=b'?', max_length=1, verbose_name='Type Category', choices=[(b'?', b'Unknown'), (b'U', b'User to User'), (b'S', b'System to User'), (b'A', b'Admin to User'), (b'P', b'User to Admin'), (b'T', b'System to Translator')])),
                ('enabled', models.BooleanField(default=False)),
                ('default_hide', models.BooleanField(default=False)),
                ('default_email', models.BooleanField(default=False)),
                ('group', models.ForeignKey(verbose_name='Limit to Group', blank=True, to='auth.Group', null=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        #DeleteIfExists('alerts_message'),
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('subject', models.CharField(max_length=128)),
                ('body', models.TextField(blank=True, null=True, verbose_name='Message Body', validators=[django.core.validators.MaxLengthValidator(8192)])),
                ('created', models.DateTimeField(default=django.utils.timezone.now)),
                ('recipient', models.ForeignKey(related_name='messages', to=settings.AUTH_USER_MODEL)),
                ('reply_to', models.ForeignKey(related_name='replies', blank=True, to='alerts.Message', null=True)),
                ('sender', models.ForeignKey(related_name='sent_messages', to=settings.AUTH_USER_MODEL)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        #DeleteIfExists('alerts_useralert'),
        migrations.CreateModel(
            name='UserAlert',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('viewed', models.DateTimeField(null=True, blank=True)),
                ('deleted', models.DateTimeField(null=True, blank=True)),
                ('alert', models.ForeignKey(related_name='sent', to='alerts.AlertType')),
                ('user', models.ForeignKey(related_name='alerts', to=settings.AUTH_USER_MODEL)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        #DeleteIfExists('alerts_useralertobject'),
        migrations.CreateModel(
            name='UserAlertObject',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=32)),
                ('o_id', models.PositiveIntegerField(null=True)),
                ('alert', models.ForeignKey(related_name='objs', to='alerts.UserAlert')),
                ('table', models.ForeignKey(blank=True, to='contenttypes.ContentType', null=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        #DeleteIfExists('alerts_useralertsetting'),
        migrations.CreateModel(
            name='UserAlertSetting',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('hide', models.BooleanField(default=True, verbose_name='Hide Alerts')),
                ('email', models.BooleanField(default=False, verbose_name='Send Email Alert')),
                ('alert', models.ForeignKey(related_name='settings', to='alerts.AlertType')),
                ('user', models.ForeignKey(related_name='alert_settings', to=settings.AUTH_USER_MODEL)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='UserAlertValue',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=32)),
                ('target', models.CharField(max_length=255)),
                ('alert', models.ForeignKey(related_name='values', to='alerts.UserAlert')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='alertsubscription',
            name='alert',
            field=models.ForeignKey(related_name='subscriptions', to='alerts.AlertType'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='alertsubscription',
            name='user',
            field=models.ForeignKey(related_name='alert_subscriptions', to=settings.AUTH_USER_MODEL),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('alerts', '0004_auto_20161024_2027'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='alerttype',
            name='category',
        ),
        migrations.RemoveField(
            model_name='alerttype',
            name='default_hide',
        ),
        migrations.RemoveField(
            model_name='alerttype',
            name='private',
        ),
        migrations.RemoveField(
            model_name='useralertsetting',
            name='hide',
        ),
        migrations.AddField(
            model_name='alerttype',
            name='default_batch',
            field=models.CharField(blank=True, max_length=1, null=True, choices=[(None, 'Instant'), (b'D', 'Daily'), (b'W', 'Weekly'), (b'M', 'Monthly')]),
        ),
        migrations.AddField(
            model_name='alerttype',
            name='default_irc',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='useralertsetting',
            name='batch',
            field=models.CharField(choices=[(None, 'Instant'), (b'D', 'Daily'), (b'W', 'Weekly'), (b'M', 'Monthly')], max_length=1, blank=True, help_text='Save all alerts and send as one email, only effects email alerts.', null=True, verbose_name='Batch Alerts'),
        ),
        migrations.AddField(
            model_name='useralertsetting',
            name='irc',
            field=models.BooleanField(default=False, help_text='If online, this alert will be sent to my irc nickname.', verbose_name='Send IRC Alert'),
        ),
        migrations.AddField(
            model_name='useralertsetting',
            name='owner',
            field=models.BooleanField(default=True, help_text='Send the alert if it sent directly to me or my group.', verbose_name='Direct Messages'),
        ),
        migrations.AlterField(
            model_name='useralertsetting',
            name='email',
            field=models.BooleanField(default=False, help_text='Send the alert to the account email address.', verbose_name='Send Email Alert'),
        ),
    ]



#
# Copyright 2014-2017, Martin Owens <doctormo@gmail.com>
#
# django-boxed-alerts is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-boxed-alerts is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with django-boxed-alerts.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Inherit from these classes if you want to create your own alert connections.
"""
__all__ = ['BaseAlert', 'EditedAlert', 'CreatedAlert', 'AddedAlert', 'RemovedAlert']

import os
import logging
from signal import SIGUSR1
from collections import defaultdict

from django.conf import settings
from django.db.models import signals as django_signals
from django.core.mail.message import EmailMultiAlternatives
from django.contrib.auth.models import Group
from django.contrib.auth import get_user_model
from alerts.template_tools import has_template, render_template, render_directly

class Lookup(object): # pylint: disable=too-few-public-methods
    """This descriptor is a class and object property, this is so
       we can do things like:

       Message.subscriptions.all()
       msgobj.subscriptions.all()
    """
    def __init__(self, base, model):
        self.base = base
        self.model = model

    def __get__(self, obj, klass=None):
        """Get a normal manager, but add in an extra attributes"""
        manager = type(self.model.objects)()
        manager.target = obj
        manager.model = self.model
        manager.alert_type = self.base.alert_type
        return manager

class BaseAlert(object): # pylint: disable=too-many-public-methods
    """None model parent class for your alert signals"""
    # Control how subscriptions will work.
    subscribe_all = True
    subscribe_any = True
    subscribe_own = True # Related users and groups

    # These defaults control how messages should be displayed or
    # Send via email to the user's email address.
    default_email = True
    default_batch = None  # Instant
    default_irc = False

    signal = django_signals.post_save
    sender = None

    # What lookup should be attached to the sender model;
    # much like ForeignKey related_name it creates reverse lookups.
    related_name = 'alerts'
    related_sub = 'subscriptions'

    subject = "{{ instance }}"
    email_subject = "{% trans 'Website Alert:' %} {{ instance }}"
    email_footer = 'alerts/alert/email_footer.txt'
    object_name = "{{ instance }}"

    # User specifies which user attribute on the instance alerts are sent
    alert_user = '.'
    alert_group = '.'

    # Should we load only on test suite run.
    test_only = False

    # Should settings be disabled and the defaults for all
    show_settings = True

    # Target is the attribute on the instance which subscriptions are bound
    target_field = None

    # Optional filter data that can be populated by the settings (e.g. language).
    filter_field = None
    filter_subscriber = None

    # A record of all the instances, and their dictionaries
    _singletons = defaultdict(dict)
    instances = {}
    slug = None

    def __init__(self, slug=None, is_test=False, is_migrate=False, **kwargs):
        # Borg singleton pattern
        self.__dict__ = self._singletons[slug]
        self.instances[slug] = self

        if type(self).slug is not None:
            # Already initialised previously
            return

        type(self).slug = slug
        self.is_test = is_test
        self.is_migrate = is_migrate
        # If signal is set to none, then we don't need to connect
        self.connected = not self.signal

        # Check the setup of this alert class
        if not self.sender:
            raise AttributeError("%s required 'sender' attribute" %
                                 type(self).__name__)
        if hasattr(self.sender, self.related_name):
            raise AttributeError("%s already has '%s' reverse lookup" % (
                self.sender.__name__, self.related_name))

        super(BaseAlert, self).__init__(**kwargs)

        from alerts.models import AlertType
        if not self.is_migrate:
            # Create an AlertType object which mirrors this class
            (self._alert_type, _) = AlertType.objects.get_or_create(
                slug=self.slug,
                values={
                    'enabled': True,
                    'default_email' : self.default_email,
                    'default_irc': self.default_irc,
                    'default_batch': self.default_batch,
                }
            )
        self.connect_signals()

        from alerts.models import UserAlert, AlertSubscription
        setattr(self.sender, self.related_name, Lookup(self, UserAlert))
        setattr(self.sender, self.related_sub, Lookup(self, AlertSubscription))

    def show_settings_now(self, user):
        """Shows the settings for these alert types"""
        return user and self.show_settings

    def connect_signals(self):
        """Attempts to start the signals late"""
        if not self.connected:
            self.signal.connect(self.call, sender=self.sender, dispatch_uid=self.slug)
            self.connected = True
        return self

    def disconnect_signals(self):
        """Disconnects the alert signal, useful for tests"""
        if self.connected:
            self.signal.disconnect(self.call, sender=self.sender, dispatch_uid=self.slug)
            self.connected = False
        return self

    @property
    def alert_type(self):
        """For tests we have to load alert_type object's late."""
        if not hasattr(self, '_alert_type'):
            # This will fail is the test doesn't include a fixture with all the right
            # alert types available. It depends on the test to populate the database.
            from alerts.models import AlertType
            self._alert_type = AlertType.objects.get(slug=self.slug)
        return self._alert_type

    @classmethod
    def get_alert_type(cls):
        """Return alert_type when we only have the alert class"""
        return cls(cls.slug).alert_type

    def get_alert_users(self, instance):
        """Returns a user of a list of users to send emails to"""
        return getattr(instance, self.alert_user, None)

    def get_alert_groups(self, instance):
        """Returns a group or a list of groups to send emails to"""
        return getattr(instance, self.alert_group, None)

    @classmethod
    def bellow(cls, **kwargs):
        """
        Manually calls this notification and sends out emails even if no
        django signal was actually triggered.
        """
        return cls(cls.slug).call(**kwargs)

    def call(self, instance, **kwargs):
        """Connect this method to the post_save signal and it will
           create an alert when the sender edits any object."""
        kwargs['instance'] = instance

        def send_to(recipient, kind=None):
            """Look through recipient and send to each"""
            if kind is None or isinstance(recipient, kind):
                ret = self.alert_type.send_to(recipient, **kwargs)
                if ret and isinstance(ret, list):
                    return ret
            elif isinstance(recipient, list):
                ret = []
                for item in recipient:
                    ret += send_to(item, kind=kind)
            return []

        users = []

        if self.subscribe_own:
            users += send_to(self.get_alert_users(instance), get_user_model())
            users += send_to(self.get_alert_groups(instance), Group)

        if self.subscribe_all:
            for sub in self.alert_type.subscriptions.filter(target__isnull=True):
                if self._filter_subscriber(sub.user, kwargs):
                    users += send_to(sub.user)

        if self.subscribe_any:
            target = instance
            if self.target_field:
                target = getattr(instance, self.target_field)

            for sub in self.alert_type.subscriptions.filter(target=target.pk):
                if self._filter_subscriber(sub.user, kwargs):
                    users += send_to(sub.user)

        return self.post_send(*users, **kwargs)

    def get_filter_field(self, user, kwargs): # pylint: disable=unused-argument
        """Returns the field that filters these objects for this user."""
        return self.filter_field

    def _filter_subscriber(self, user, kwargs):
        field = self.get_filter_field(user, kwargs)
        if field:
            from alerts.models import UserAlertSetting
            try:
                setting = user.alert_settings.get(alert__slug=self.slug)
            except UserAlertSetting.DoesNotExist:
                setting = None
            return self.get_filter_subscriber(user, setting, kwargs)
        return True

    def get_filter_subscriber(self, user, setting, kwargs): # pylint: disable=unused-argument
        """Do actual filtering of the user for this object"""
        return self.filter_subscriber

    @staticmethod
    def post_send(*users, **_):
        """Complete the call and tidy up if needed"""
        return bool(users)

    @property
    def instance_type(self):
        """Returns the type of the instance, either by sender or the target"""
        if self.target_field:
            return self.sender._meta.get_field(self.target_field).rel.to # pylint: disable=protected-access
        return self.sender

    @property
    def template(self):
        """Returns the html template for website visible alerts"""
        template = "%s/alert/%s.html" % tuple(self.slug.split('.', 1))
        if has_template(template):
            return template
        return "alerts/type_default.html"

    @property
    def email_template(self):
        """Returns the template for the email"""
        template = "%s/alert/email_%s.txt" % tuple(self.slug.split('.', 1))
        if has_template(template):
            return template
        else:
            logging.warning("Template: '%s' doesn't exist.", template)
        return "alerts/email_default.txt"

    @property
    def name(self):
        """The name is a human visible name used to show users"""
        raise NotImplementedError("Name is a required property for alerts.")

    @property
    def desc(self):
        """Describe the alert, what it does, and how it's triggered to users"""
        raise NotImplementedError("Desc is a required property for alerts.")

    @property
    def info(self):
        """Info is like description but provides detailed information"""
        raise NotImplementedError("Info is a required property for alerts.")

    def get_object(self, **fields):
        """Returns object matching field using Alert's target object type"""
        model = self.sender
        if self.target_field:
            # Get FoeignKey's mdel linking this alert instead
            model = getattr(model, self.target_field).field.rel.to
        return model.objects.get(**fields)

    def get_object_name(self, obj):
        """Return's a label for this kind of subscription for an object"""
        if obj:
            return render_directly(self.object_name, {'object': obj})
        return None

    @staticmethod
    def format_data(data):
        """Overridable function to format data for the template"""
        return data

    def get_subject(self, context_data):
        """Return's the subject text given the object"""
        return render_directly(self.subject, self.format_data(context_data))

    def get_body(self, context_data):
        """Returns the rendered body using the context_data dictionary"""
        return render_template(self.template, context_data)

    def get_email_body(self, context_data):
        """Returns the email body text rendered from the context_data"""
        return render_template(self.email_template, context_data)

    @staticmethod
    def send_irc_msg(user):
        """
        Send the running IRC bot a kick up the bum about a new alert.
        """
        try:
            with open(settings.IRCBOT_PID, 'r') as pid:
                pid = int(pid.read().strip())
            with open('/proc/%d/cmdline' % pid, 'r') as proc:
                assert('ircbot' in proc.read())
            os.kill(pid, SIGUSR1)
            return bool(user)
        except: # pylint: disable=bare-except
            # Any errors may mean:
            # * IRCBOT not configured, not running
            # * IRCBOT pid file exists but process isn't ircbot
            # * We don't have permission to signal process
            return False

    def send_email(self, recipient, context_data, **kwargs):
        """Sends the email to the recipient using the context_data"""
        if not recipient:
            return False

        context_data = self.format_data(context_data)
        subject = render_directly(self.email_subject, context_data)

        kwargs.update({
            'subject': subject.strip().replace('\n', ' ').replace('\r', ' '),
            'body': self.get_email_body(context_data),
            'to': [recipient],
        })

        if self.email_footer is not None:
            kwargs['body'] += render_template(self.email_footer, context_data)

        #if self.alert_type.CATEGORY_USER_TO_USER:
        #    this doesn't work yet because we don't know how to get the sender
        #    kwargs['reply_to'] = self.get_sender(context_data)

        # This will fail silently if not configured right
        return EmailMultiAlternatives(**kwargs).send(True)


class EditedAlert(BaseAlert): # pylint: disable=abstract-method
    """Special alert type, when any item is edited"""
    def call(self, instance=None, **kwargs):
        if not kwargs.get('created', False):
            return super(EditedAlert, self).call(instance=instance, **kwargs)
        return False


class CreatedAlert(BaseAlert): # pylint: disable=abstract-method
    """Special alert type, when any item is created"""
    def call(self, instance=None, **kwargs):
        if kwargs.get('created', False):
            return super(CreatedAlert, self).call(instance=instance, **kwargs)
        return False

class AddedAlert(BaseAlert):
    """Special alert type, when an item is added to a many to many relationship"""
    signal = django_signals.m2m_changed
    m2m_action = 'post_add'
    m2m_reverse = False
    sender = property(lambda self: self.m2m_sender.through)

    @property
    def m2m_sender(self):
        """Returns the sender field that the many to many is attached to"""
        raise NotImplementedError("Many to Many sender must be specified")

    def call(self, pk_set=None, instance=None, action=None, reverse=False, **kwargs): # pylint: disable=arguments-differ
        if action == self.m2m_action and reverse == self.m2m_reverse:
            att = self.m2m_sender.field.attname
            kwargs['items'] = getattr(instance, att).filter(pk__in=pk_set)
            return super(AddedAlert, self).call(instance=instance, **kwargs)
        return False


class RemovedAlert(AddedAlert): # pylint: disable=abstract-method
    """Special alert type, when an item is removed from a many to many relationship"""
    m2m_action = 'pre_remove'


class CustomAlert(BaseAlert): # pylint: disable=abstract-method
    """Will not bind a singal and instead cls.call(obj) should be called manually"""
    signal = None

    @classmethod
    def call(cls, instance, **kw):
        alert = cls(cls.slug)
        return super(CustomAlert, alert).call(instance=instance, **kw)



#
# Copyright 2015, Martin Owens <doctormo@gmail.com>
#
# This file is part of the software inkscape-web, consisting of custom 
# code for the Inkscape project's django-based website.
#
# inkscape-web is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# inkscape-web is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with inkscape-web.  If not, see <http://www.gnu.org/licenses/>.
#

from autotest.base import ExtraTestCase
from alerts.models import Message, AlertType, UserAlertManager
from alerts.alert import MessageAlert

from person.models import User

class BasicTests(ExtraTestCase):
    fixtures = ['test-auth']

    def test_registration(self):
        """Message alerts have been registered"""
        self.assertTrue(hasattr(Message, 'alerts'))

    def test_links(self):
        """Messages have alert manager has AlertBase has AlertType"""
        alerts = Message.alerts
        self.assertEqual(type(alerts), UserAlertManager)
        self.assertEqual(type(alerts.alert_type), AlertType)
        self.assertEqual(type(alerts.alert_type._alerter), MessageAlert)
        self.assertEqual(alerts.alert_type.sender, Message)

    def test_object_link(self):
        """Objects have working manager"""
        message = Message.objects.create(recipient_id=1, sender_id=1)
        alerts = message.alerts
        self.assertEqual(type(alerts), UserAlertManager)
        self.assertEqual(type(alerts.target), Message)

    def test_user_link(self):
        """Users have a list of alerts"""
        user = User.objects.get(pk=1)
        self.assertEqual(user.alerts.count(), 0)
        message = Message.objects.create(recipient=user, sender_id=2)
        self.assertEqual(user.alerts.count(), 1)


class AlertUserTests(ExtraTestCase):
    fixtures = ['test-auth', 'test-messages']
    credentials = dict(username='tester', password='123456')
    alert_type = 'alerts.test_message_alert'

    def test_00_fixture_messages(self):
        self.assertEqual(Message.objects.count(), 3)
        for message in Message.objects.all():
            self.assertEqual(message.alerts.count(), 1)
            self.assertEqual(message.test_alerts.count(), 1)
        user = User.objects.get(pk=3)
        self.assertEqual(user.alerts.count(), 3)
        user = User.objects.get(pk=2)
        self.assertEqual(user.alerts.count(), 3)

    def assertMessageCount(self, cls, obj):
        """Test class based and object based lookups"""
        self.assertEqual(Message.subscriptions.count(), cls)
        self.assertEqual(Message.objects.get(pk=1).subscriptions.count(), obj)

    def test_subscribe_class(self):
        self.assertMessageCount(0, 0)
        response = self.assertGet('alert.subscribe', slug=self.alert_type)
        self.assertContains(response, "Subscribe to All Personal Message")

        response = self.assertPost('alert.subscribe', slug=self.alert_type)
        self.assertContains(response, "Subscription created")

        self.assertTrue(hasattr(Message, 'subscriptions'))
        self.assertEqual(Message.subscriptions.count(), 1)

        sub = Message.subscriptions.get()
        self.assertEqual(sub.alert.slug, self.alert_type)
        self.assertEqual(sub.user.username, 'tester')
        self.assertEqual(sub.target, None)
        self.assertMessageCount(1, 0)

    def test_subscribe_item(self):
        self.assertMessageCount(0, 0)

        response = self.assertGet('alert.subscribe', slug=self.alert_type, pk=1)
        self.assertContains(response, "Subscribe to Each message themselves for testing")

        response = self.assertPost('alert.subscribe', slug=self.alert_type, pk=1)
        self.assertContains(response, "Subscription created")

        self.assertTrue(hasattr(Message, 'subscriptions'))

        subs = Message.objects.get(pk=1).subscriptions
        self.assertEqual(subs.count(), 1)
        self.assertEqual(subs.get().alert.slug, self.alert_type)
        self.assertEqual(subs.get().user.username, 'tester')
        self.assertEqual(subs.get().target, 1)

        self.assertMessageCount(0, 1)

    def test_subdcribe_item_removed(self):
        pass

    def test_create_alert(self):
        pass

    def test_not_subscribed(self):
        pass

    def test_unsubscribe(self):
        pass

    def test_list_alerts(self):
        response = self.assertGet('alert.category', slug='alerts.message_alert')
        self.assertContains(response, '"')
        self.assertContains(response, 'id="alert_')
        # Test does not contains different alert type (test alert type?)

    def test_list_all_alerts(self):
        response = self.assertGet('alerts')
        self.assertContains(response, 'Subject One')
        self.assertContains(response, 'Subject Two')
        self.assertContains(response, 'Subject Three')

    def test_view_alert(self):
        pass

    def test_delete_alert(self):
        pass

    def test_settings_list(self):
        pass


class MessageOnlyTests(ExtraTestCase):
    def test_sent_list(self):
        pass




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('alerts', '0003_auto_20150827_0424'),
    ]

    operations = [
        migrations.AlterField(
            model_name='alerttype',
            name='slug',
            field=models.CharField(unique=True, max_length=32, verbose_name='URL Slug'),
        ),
    ]



#!/usr/bin/env python
#
# Copyright (C) 2017 Martin Owens
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
# pylint: disable=bad-whitespace

from setuptools import setup, find_packages

# Grab description for Pypi
with open('README.md') as fhl:
    DESC = fhl.read()

setup(
    name             = "django-boxed-alerts",
    version          = "1.2.9",
    description      = "Robust user emailing and notification system for django web framework.",
    long_description = DESC,
    author           = 'Martin Owens',
    url              = 'https://gitlab.com/doctormo/django-boxed-alerts',
    author_email     = 'doctormo@gmail.com',
    platforms        = 'linux',
    license          = 'AGPLv3',
    install_requires = ['django>=1.11,<2.0',],
    packages         = find_packages(),
    include_package_data=True,
    package_dir={
        'alerts': 'alerts',
    },
    classifiers      = [
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
    ],
)



#
# Copyright 2016, Martin Owens <doctormo@gmail.com>
#
# This file is part of the software inkscape-web, consisting of custom 
# code for the Inkscape project's django-based website.
#
# inkscape-web is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# inkscape-web is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with inkscape-web.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Forms for the alert system 
"""

from django.forms import *
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse

from inkscape.utils import to
from .models import Message, AlertType, AlertSubscription, UserAlertSetting

class MessageForm(ModelForm):
    reply_to = IntegerField(widget=HiddenInput, required=False)
    recipient = IntegerField(widget=HiddenInput)

    class Meta:
        model = Message
        fields = ('subject','body', 'recipient', 'reply_to')

    def clean_recipient(self):
        pk = self.cleaned_data['recipient']
        return get_user_model().objects.get(pk=pk)

    def clean_reply_to(self):
        pk = self.cleaned_data['reply_to']
        if pk:
            return Message.objects.get(pk=pk)


class SettingsBaseFormSet(BaseModelFormSet):
    def __init__(self, instance, *args, **kw):
        self.user = instance
        super(SettingsBaseFormSet, self).__init__(*args, **kw)

    @to(list)
    def get_queryset(self):
        """Return a fixed list of alert_settings (not a queryset)"""
        for alert in AlertType.objects.all().order_by('slug'):
            if alert.show_settings_now(self.user):
                yield alert.settings.for_user(self.user)

    def _construct_form(self, i, **kwargs):
        # Link POST forms to their alert_types, so override when is_bound.
        if self.is_bound and i < self.initial_form_count():
            alert_id = self.data["%s-%s" % (self.add_prefix(i), 'alert')]
            for obj in self.get_queryset():
                if str(obj.alert_id) == str(alert_id):
                    kwargs['instance'] = obj
            # Note use of parent's parent construct_form call here.
            return super(BaseModelFormSet, self)._construct_form(i, **kwargs)
        return super(SettingsBaseFormSet, self)._construct_form(i, **kwargs)

    @property
    def media(self):
        """We want all the forms media added together"""
        media = super(SettingsBaseFormSet, self).media
        for form in self.forms[1:]:
            media += form.media
        return media



class SettingsForm(ModelForm):
    suball = BooleanField(required=False, label=_("Subscribe to All"),
        help_text=_("<strong>Warning!</strong> This can result in a lot of messages."))
    em_msg = _("<strong>Warning!</strong> You haven't told us your mail addres"
               "s yet! If you would like to receive emails from inkscape.org, "
               "enter an email address <a href='%(url)s'>in your profile</a>.")

    class Meta:
        model = UserAlertSetting
        fields = ('alert', 'email', 'irc', 'batch', 'owner', 'suball', 'filter_value')

    def __init__(self, *args, **kw):
        super(SettingsForm, self).__init__(*args, **kw)
        self.fields['alert'].widget = HiddenInput()
        self.subs = self.instance.subscriptions

        # XXX Replace with "disabled" class and change help_text
        if not self.instance.user.ircnick:
            self.fields.pop('irc')
        if not self.instance.user.email:
            self.fields['email'].widget.attrs['disabled'] = 'disabled'
            self.fields['email'].help_text = self.em_msg % {'url': reverse('edit_profile')}
        if not self.instance.alert.subscribe_own:
            self.fields.pop('owner')
            # XXX the name of this field needs to change depending on if subscribe is visible at all.
        if self.instance.alert.subscribe_all:
            self.suball = self.subs.filter(target__isnull=True)
            self.fields['suball'].initial = self.suball.count() == 1
        else:
            self.fields.pop('suball')
        # Apply the filter field widget/field as needed
        filter_field = self.instance.alert.get_filter_field(self.instance.user, {})
        if filter_field:
            self.fields['filter_value'] = filter_field
            self.fields['filter_value'].required = False
        else:
            self.fields.pop('filter_value')

        if self.instance.alert.subscribe_any:
            for sub in self.subs.filter(target__isnull=False):
                label = _("Subscription to %s") % str(sub.object())
                field = BooleanField(required=False, label=label)
                field.delete = True
                self.fields['delete_%d' % sub.pk] = field

    def save(self, commit=True, **kw):
        ret = super(SettingsForm, self).save(commit=commit, **kw)
        if commit:
            if self.instance.alert.subscribe_all:
                if self.cleaned_data.get('suball', False):
                    self.subs.get_or_create(None, alert_id=self.instance.alert_id, user_id=self.instance.user_id)
                else:
                    self.subs.filter(target__isnull=True).delete()
            if self.instance.alert.subscribe_any:
                for obj in self.subs.filter(target__isnull=False):
                    obj.delete()
        return ret

    @property
    def label(self):
        if not self.instance.pk:
            return str(self.instance) + "*"
        return str(self.instance)

    @property
    def description(self):
        return self.instance.alert.info


SettingsFormSet = modelformset_factory(
    model=UserAlertSetting,
    form=SettingsForm,
    formset=SettingsBaseFormSet, extra=0)

